import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

public class ImageMap extends JPanel {
    JavaMapper frame;

    Image image,offScreenImage;
    Graphics s;
    int imageW,imageH;
    boolean imageLoaded;

    String mapName,defaultURL,imagePath,imageFileName,saveFilePath,saveFileType="client";
    boolean saveNames;
    String toolMode="rect";

    Timer polyTimer;
    Tool tool;

    public ImageMap(Image image, String imagePath, JavaMapper frame) {
	this.image = image;
	this.imagePath = imagePath;
	this.frame = frame;
	
	this.imageFileName = imagePath.substring(imagePath.lastIndexOf("/")+1);
	this.mapName = imageFileName.substring(0,imageFileName.indexOf("."));
	this.defaultURL = "";
	
	imageW = image.getWidth(this);
	imageH = image.getHeight(this);
	frame.println("Recieved "+imagePath+" ("+String.valueOf(imageW)+"x"+String.valueOf(imageH)+")");
	setPreferredSize(new Dimension(imageW,imageH));

	tool = new Tool(this);
	addMouseListener(tool);
	addMouseMotionListener(tool);
	polyTimer = new Timer(this);
	polyTimer.start();

	imageLoaded = true;
	repaint();
    }

    public void paint(Graphics g) {
	update(g);
    }
    public void update(Graphics g) {
	super.paintComponent(g);
	
	Dimension d = getSize();
	offScreenImage = createImage(d.width,d.height);
	s = offScreenImage.getGraphics();
	
	s.setColor(new Color(192,192,192));
	s.fillRect(0,0,d.width,d.height);
	
	if (imageLoaded) {
	    s.drawImage(image,0,0,this);
	    tool.draw(s);
	}
	
	g.drawImage(offScreenImage,0,0,this);
    }

    public void setTool (String mode) {
	toolMode = mode;
	if (toolMode=="select") setNewCursor(Cursor.DEFAULT_CURSOR);
	else setNewCursor(Cursor.CROSSHAIR_CURSOR);
	tool.endEverything();
    }
	
    public void setNewCursor (int cursor) {
	if (cursor != getCursor().getType()) {
	    setCursor(new Cursor(cursor));
	}
    }

    public String getCode(String type) {
	tool.remakeShapes();
	String code = "";
	if (type=="client") {
	    code = "<map name=\""+mapName+"\">\n";
	    if (defaultURL.length()>0) code += "<area default=\""+defaultURL+"\">\n";
	    for (int i=0;i<tool.nShapes;i++) {
		code += "<area ";
		if (saveNames) code += "name=\""+tool.shapes[i].getName()+"\" ";
		code += "shape=\""+tool.shapes[i].getType()+"\" coords=\""+tool.shapes[i].getCoords(type)+"\" href=\""+tool.shapes[i].getURL()+"\">\n";
	    }
	    code += "</map>\n";
	    code += "<img src=\""+imageFileName+"\" usemap=\"#"+mapName+"\" width="+imageW+" height="+imageH+">\n";
	}
	else if (type=="server") {
	    if (defaultURL.length()>0) code += "default "+defaultURL+"\n";
	    for (int i=0;i<tool.nShapes;i++) {
		code += tool.shapes[i].getType()+" "+tool.shapes[i].getURL()+" "+tool.shapes[i].getCoords(type)+"\n";
	    }
	}
	return code;
    }


    // -- Tool Class --------------------------------------
    
    class Tool extends MouseAdapter implements MouseMotionListener {
	int offsetX,offsetY,mouseX,mouseY;
	
	boolean drawActive,highlightActive,selectActive,dragActive;
	
	Rect rect[] = new Rect[200];
	int nRect;
	
	Circle circle[] = new Circle[200];
	int nCircle;
	
	Poly p,poly[] = new Poly[200];
	int nPoly;
		
	ShapeInterface currentShape,testShape,shapes[] = new ShapeInterface[600];
	int nShapes;
	ImageMap map;
	JavaMapper frame;
	
	Color color = Color.blue;
	
	public Tool(ImageMap map) {
	    this.map = map;
	    this.frame = map.frame;
	}

	public void createNewRect(int x1, int y1, int x2, int y2) {
	    int i = nRect;
	    rect[i] = new Rect(x1,y1);
	    rect[i].setIndex(i);
	    rect[i].newPoint(x2,y2);
	    currentShape = rect[nRect++];
	    repaint();
	}

	public void createNewCircle(int x1, int y1, int x2, int y2) {
	    int i = nCircle;
	    circle[i] = new Circle(x1,y1);
	    circle[i].setIndex(i);
	    circle[i].newPoint(x2,y2);
	    currentShape = circle[nCircle++];
	    repaint();
	}

	public void createNewPoly(Vector vx, Vector vy) {
	    int i = nPoly;
	    int x1 = ((Integer)(vx.elementAt(0))).intValue();
	    int y1 = ((Integer)(vy.elementAt(0))).intValue();
	    poly[i] = new Poly(x1,y1);
	    poly[i].setIndex(i);
	    for (int j=1;j<vx.size();j++) {
		int x2 = ((Integer)(vx.elementAt(j))).intValue();
		int y2 = ((Integer)(vy.elementAt(j))).intValue();
		poly[i].newPoint(x2,y2);
	    }
	    poly[i].setDrawing(false);
	    currentShape = poly[nPoly++];
	    repaint();
	}
	
	public void startDraw(int x, int y) {
	    if (!drawActive) {
		if (toolMode=="rect" && nRect<200) {
		    rect[nRect] = new Rect(x,y);
		    rect[nRect].setIndex(nRect);
		    currentShape = rect[nRect++];
		}
		else if (toolMode=="circle" && nCircle<200) {
		    circle[nCircle] = new Circle(x,y);
		    circle[nCircle].setIndex(nCircle);
		    currentShape = circle[nCircle++];
		}
		else if (toolMode=="poly" && nPoly<200) {
		    poly[nPoly] = new Poly(x,y);
		    poly[nPoly].setIndex(nPoly);
		    currentShape = poly[nPoly++];
		    poly[currentShape.getIndex()].nextShown = false;
		}
		if (toolMode!="select") {
		    drawActive = true;
		    frame.mDeleteAllShapes.setEnabled(true);
		}
	    }
	    else if (currentShape.getType()=="poly") {
		if (!poly[currentShape.getIndex()].checkNewPoint(x,y)) {
		    continueDraw(x,y);
		    poly[currentShape.getIndex()].nextShown = false;
		}
		else {
		    poly[currentShape.getIndex()].setDrawing(false);
		    if (poly[currentShape.getIndex()].getPoints()<=2) {
			deleteShape();
					}
		    else {
			frame.println("Created "+currentShape.getType()+" "+String.valueOf(currentShape.getIndex()));
		    }
		    currentShape = null;
		    drawActive = false;
		}
	    }
	}
	
	public void continueDraw(int x, int y) {
	    currentShape.newPoint(x,y);
	    repaint();
	}
	
	public void finishDraw() {
	    if (currentShape.getW()==0 && currentShape.getH()==0) {
		deleteShape();
	    }
	    else {
		frame.println("Created "+currentShape.getType()+" "+String.valueOf(currentShape.getIndex()));
	    }
	    currentShape = null;
	    drawActive = false;
	}
	
	public boolean highlightShape(int x, int y) {
	    testShape = null;
	    boolean shapefound;
	    for (int i=nRect-1;i>=0;i--) {
		if (rect[i].contains(x,y)) {
		    testShape = rect[i];
		    break;
		}
	    }
	    for (int i=nCircle-1;i>=0;i--) {
		if (circle[i].contains(x,y)) {
		    testShape = circle[i];
		    break;
		}
	    }
	    for (int i=nPoly-1;i>=0;i--) {
		if (poly[i].contains(x,y)) {
		    testShape = poly[i];
		    break;
		}
	    }
	    if (testShape!=null) {
		if (currentShape!=testShape && currentShape!=null) currentShape.setHighlighted(false);
		testShape.setHighlighted(true);
		if (currentShape!=testShape) repaint();
		currentShape = testShape;
		highlightActive = true;
	    }
	    else {
		if (currentShape!=null) {
		    currentShape.setHighlighted(false);
		    currentShape = null;
		    repaint();
		    highlightActive = false;
		}
	    }
	    return highlightActive;
	}
	
	public void deleteShape() {
	    frame.println("deleted "+currentShape.getType()+" "+String.valueOf(currentShape.getIndex()));
	    currentShape.setSelected(false);
	    frame.mDeleteShape.setEnabled(false);
	    frame.bDeleteShape.setEnabled(false);
	    selectActive = false;
	    if (currentShape.getType()=="rect") {
		for (int i=currentShape.getIndex();i<nRect-1;i++) {
		    rect[i] = rect[i+1];
		    rect[i].setIndex(i);
		}
		nRect -= 1;
	    }
	    if (currentShape.getType()=="circle") {
		for (int i=currentShape.getIndex();i<nCircle-1;i++) {
		    circle[i] = circle[i+1];
		    circle[i].setIndex(i);
		}
		nCircle -= 1;
	    }
	    if (currentShape.getType()=="poly") {
		for (int i=currentShape.getIndex();i<nPoly-1;i++) {
		    poly[i] = poly[i+1];
		    poly[i].setIndex(i);
		}
		nPoly -= 1;
	    }
	    if (nPoly==0 && nCircle==0 && nRect==0) {
		frame.mDeleteAllShapes.setEnabled(false);
	    }
	    currentShape = null;
	    frame.refreshDialogs();
	    repaint();
	}
	
	public void deleteAllShapes() {
	    endEverything();
	    nRect = 0;
	    nCircle = 0;
	    nPoly = 0;
	    frame.mDeleteShape.setEnabled(false);
	    frame.mDeleteAllShapes.setEnabled(false);
	    frame.refreshDialogs();
	    repaint();
	}
	
	public void selectShape() {
	    currentShape.setSelected(true);
	    frame.mDeleteShape.setEnabled(true);
	    frame.bDeleteShape.setEnabled(true);
	    highlightActive = false;
	    selectActive = true;
	}
	
	public void deselectShape() {
	    if (currentShape==null) return;
	    currentShape.setSelected(false);
	    frame.mDeleteShape.setEnabled(false);
	    frame.bDeleteShape.setEnabled(false);
	    selectActive = false;
	}
	
	public int getSelectedIndex() {
	    if (currentShape==null) return -1;
	    remakeShapes();
	    for (int i=0;i<nShapes;i++) {
		if (shapes[i]==currentShape) return i;
	    }
	    return -1;
	}
	
	public boolean startDragShape(int x, int y) {
	    if (currentShape.contains(x,y)) {
		offsetX = x-currentShape.getX();
		offsetY = y-currentShape.getY();
		dragActive = true;
		setNewCursor(Frame.MOVE_CURSOR);
		return true;
	    }
	    else return false;
	}
	
	public void dragShape(int x, int y) {
	    currentShape.moveTo(x-offsetX,y-offsetY);
	    repaint();
	}
	
	public void endEverything() {
	    if (drawActive && currentShape!=null && currentShape.getType()=="poly") {
		poly[currentShape.getIndex()].setDrawing(false);
		if (poly[currentShape.getIndex()].getPoints()<=2) {
		    deleteShape();
		}
	    }
	    if (currentShape!=null) {
		currentShape.setSelected(false);
		currentShape.setHighlighted(false);
		currentShape = null;
	    }
	    drawActive = false;
	    highlightActive = false;
	    selectActive = false;
	    repaint();
	}
	
	public void draw(Graphics s) {
	    s.setColor(color);
	    for (int i=0;i<nRect;i++){
		rect[i].draw(s);
	    }
	    for (int i=0;i<nCircle;i++){
		circle[i].draw(s);
	    }
	    for (int i=0;i<nPoly;i++){
		poly[i].draw(s);
	    }
	}
	
	public void mousePressed(MouseEvent e) {
	    int x = e.getX();
	    int y = e.getY();
	    if (highlightActive) {  // if highligted and clicked, select it
		selectShape();
		startDragShape(x,y);
	    }
	    else if (selectActive) {
		if ((currentShape.getType()=="circle" || currentShape.getType()=="rect") && currentShape.getResize(x,y)!=Frame.DEFAULT_CURSOR) {
		}
		else if (currentShape.getType()=="poly" && currentShape.getResize(x,y)!=-1) {
		    polyTimer.setInfo("poly",e,x,y,currentShape.getResize(x,y));
		    polyTimer.resume();
		}
		else if (!startDragShape(x,y)) {  // start drag or deselect
		    deselectShape();
		    if (highlightShape(x,y)) {  // check if clicked another and select/drag it
			selectShape();
			startDragShape(x,y);
		    }
		}
	    }
	    else {
		startDraw(x,y);
	    }
	    repaint();
	}
	
	public void mouseDragged(MouseEvent e) {
	    int x = e.getX();
	    int y = e.getY();
	    mouseX = x;
	    mouseY = y;
	    frame.mouseXY.setText("("+String.valueOf(x)+","+String.valueOf(y)+")");
	    if (drawActive) {
		// place another point
		if (currentShape.getType()=="poly") {
		    poly[currentShape.getIndex()].changePoint(x,y);
		    repaint();
		}
		else continueDraw(x,y);
	    }
	    else if (selectActive && currentShape.resize(x,y)) { // resize
		repaint();
	    }
	    else if (dragActive) {  // drag it
		dragShape(x,y);
	    }
	}
	
	public void mouseMoved(MouseEvent e) {
	    int x = e.getX();
	    int y = e.getY();
	    mouseX = x;
	    mouseY = y;
	    frame.mouseXY.setText("("+String.valueOf(x)+","+String.valueOf(y)+")");
	    if (toolMode=="select") {
		if (!selectActive) highlightShape(x,y);
		else if (!dragActive) {
		    if (currentShape.getType()=="poly") {
			if (currentShape.getResize(x,y)!=-1) setNewCursor(Frame.MOVE_CURSOR);
			else setNewCursor(Frame.DEFAULT_CURSOR);
		    }
		    else setNewCursor(currentShape.getResize(x,y));
		}
	    }
	    else if (drawActive && currentShape.getType()=="poly") {
		poly[currentShape.getIndex()].showNext(x,y);
		repaint();
	    }
	}
	
	public void mouseReleased(MouseEvent e) {
	    int x = e.getX();
	    int y = e.getY();
	    if (drawActive) {
		if (currentShape.getType()=="poly") {
		    poly[currentShape.getIndex()].showNext(x,y);
		    poly[currentShape.getIndex()].nextShown = true;
		}
		else finishDraw();
	    }
	    else if (dragActive) {
		setNewCursor(Frame.DEFAULT_CURSOR);
		dragActive = false;
	    }
	    frame.refreshDialogs();
	    repaint();
	}
	

	
	public void showPolyPopup() {
	    if (mouseX>=polyTimer.x-1 && mouseX<=polyTimer.x+1 && mouseY>=polyTimer.y-1 && mouseY<=polyTimer.y+1) {
		frame.polyPopup.show(polyTimer.e.getComponent(),mouseX,mouseY);
	    }
	}
	
	public void deletePolyPoint() {
	    poly[currentShape.getIndex()].deletePoint(polyTimer.node);
	    repaint();
	}
		
	public void insertPolyPoint() {
	    poly[currentShape.getIndex()].insertPoint(polyTimer.node);
	    repaint();
	}
	
	public void remakeShapes() {
	    nShapes = 0;
	    for (int i=0;i<nRect;i++) {
		shapes[nShapes++] = rect[i];
	    }
	    for (int i=0;i<nCircle;i++) {
		shapes[nShapes++] = circle[i];
	    }
	    for (int i=0;i<nPoly;i++) {
		shapes[nShapes++] = poly[i];
	    }
	}
	
    }

	
    // -- Timer Class --------------------------------------
    
    class Timer extends Thread {
	ImageMap map;
	boolean active = false;
	String command;
	int x,y,node;
	MouseEvent e;
		
	public Timer(ImageMap map) {
	    this.map = map;
	}
	
	public void setInfo(String command, MouseEvent e, int x, int y, int node) {
	    this.command = command;
	    this.e = e;
	    this.x = x;
	    this.y = y;
	    this.node = node;
	}
	
	public void run() {
	    while (true) {
		try {
		    sleep(900);
		    if (this.command=="poly") {
			map.tool.showPolyPopup();
		    }
		}
		catch (InterruptedException e) { }
		suspend();
	    }
	}
    }
   
}





